# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: ray.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\tray.proto\x12\tray.serve\"2\n\x11ModelReadyRequest\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x0f\n\x07version\x18\x02 \x01(\t\"#\n\x12ModelReadyResponse\x12\r\n\x05ready\x18\x01 \x01(\x08\"5\n\x14ModelMetadataRequest\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x0f\n\x07version\x18\x02 \x01(\t\"<\n\x0bInferTensor\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x10\n\x08\x64\x61tatype\x18\x02 \x01(\t\x12\r\n\x05shape\x18\x03 \x03(\x03\"\x8e\x02\n\x15ModelMetadataResponse\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x10\n\x08versions\x18\x02 \x03(\t\x12\x11\n\tframework\x18\x03 \x01(\t\x12?\n\x06inputs\x18\x04 \x03(\x0b\x32/.ray.serve.ModelMetadataResponse.TensorMetadata\x12@\n\x07outputs\x18\x05 \x03(\x0b\x32/.ray.serve.ModelMetadataResponse.TensorMetadata\x1a?\n\x0eTensorMetadata\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x10\n\x08\x64\x61tatype\x18\x02 \x01(\t\x12\r\n\x05shape\x18\x03 \x03(\x03\"\xf8\x01\n\x11ModelInferRequest\x12\x12\n\nmodel_name\x18\x01 \x01(\t\x12\x15\n\rmodel_version\x18\x02 \x01(\t\x12&\n\x06inputs\x18\x03 \x03(\x0b\x32\x16.ray.serve.InferTensor\x12H\n\x07outputs\x18\x06 \x03(\x0b\x32\x37.ray.serve.ModelInferRequest.InferRequestedOutputTensor\x12\x1a\n\x12raw_input_contents\x18\x07 \x03(\x0c\x1a*\n\x1aInferRequestedOutputTensor\x12\x0c\n\x04name\x18\x01 \x01(\t\"\x85\x01\n\x12ModelInferResponse\x12\x12\n\nmodel_name\x18\x01 \x01(\t\x12\x15\n\rmodel_version\x18\x02 \x01(\t\x12\'\n\x07outputs\x18\x05 \x03(\x0b\x32\x16.ray.serve.InferTensor\x12\x1b\n\x13raw_output_contents\x18\x06 \x03(\x0c\x32\xfc\x01\n\nRayService\x12K\n\nModelReady\x12\x1c.ray.serve.ModelReadyRequest\x1a\x1d.ray.serve.ModelReadyResponse\"\x00\x12T\n\rModelMetadata\x12\x1f.ray.serve.ModelMetadataRequest\x1a .ray.serve.ModelMetadataResponse\"\x00\x12K\n\nModelInfer\x12\x1c.ray.serve.ModelInferRequest\x1a\x1d.ray.serve.ModelInferResponse\"\x00\x42\x07Z\x05./rayb\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'ray_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:
  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'Z\005./ray'
  _globals['_MODELREADYREQUEST']._serialized_start=24
  _globals['_MODELREADYREQUEST']._serialized_end=74
  _globals['_MODELREADYRESPONSE']._serialized_start=76
  _globals['_MODELREADYRESPONSE']._serialized_end=111
  _globals['_MODELMETADATAREQUEST']._serialized_start=113
  _globals['_MODELMETADATAREQUEST']._serialized_end=166
  _globals['_INFERTENSOR']._serialized_start=168
  _globals['_INFERTENSOR']._serialized_end=228
  _globals['_MODELMETADATARESPONSE']._serialized_start=231
  _globals['_MODELMETADATARESPONSE']._serialized_end=501
  _globals['_MODELMETADATARESPONSE_TENSORMETADATA']._serialized_start=438
  _globals['_MODELMETADATARESPONSE_TENSORMETADATA']._serialized_end=501
  _globals['_MODELINFERREQUEST']._serialized_start=504
  _globals['_MODELINFERREQUEST']._serialized_end=752
  _globals['_MODELINFERREQUEST_INFERREQUESTEDOUTPUTTENSOR']._serialized_start=710
  _globals['_MODELINFERREQUEST_INFERREQUESTEDOUTPUTTENSOR']._serialized_end=752
  _globals['_MODELINFERRESPONSE']._serialized_start=755
  _globals['_MODELINFERRESPONSE']._serialized_end=888
  _globals['_RAYSERVICE']._serialized_start=891
  _globals['_RAYSERVICE']._serialized_end=1143
# @@protoc_insertion_point(module_scope)
